/*******************************************************************************
*                                                                              *
* System      : SOIPIX readout module                                          *
* Block       : Example for the User FPGA                                      *
* Module      : SOIPIX_USER                                                    *
* Version     : v 0.0.1 2008/11/05                                             *
*                                                                              *
* Description : Top Module                                                     *
*                                                                              *
* Designer    : Minoru Hirose & Tomohisa Uchida                                *
*                                                                              *
*                Copyright (c) 2008 Minoru Hirose & Tomohisa Uchida            *
*                All rights reserved                                           *
*                                                                              *
*******************************************************************************/
`define FPGA_VER 32'h0805_1901
`define FPGA_ID  32'h0005_0000

//`include "../Src/sys_dcm.v"
//`include "../Src/SIO_SLAVE.v"
//`include "../Src/SIO_REG.v"

module SOIPIX_USER(
// System
	OSC					,	// in	: 50MHz clock
	SW_RSTn				,	// in	: System reset
// User I/F
	USR_CLK				,	// out	: Clock
	USR_ACTIVE			,	// in	: TCP established
	USR_CLOSE_REQ		,	// in	: Request to close a TCP Connection
	USR_CLOSE_ACK		,	// out	: Acknowledge for USR_CLOSE_REQ
	// TCP Tx
	USR_TX_AFULL		,	// in	: Almost full flag of a TCP Tx FIFO
	USR_TX_WE			,	// out	: TCP Tx Data write enable
	USR_TX_WD			,	// out	: TCP Tx Data[7:0]
	// TCP Rx
	USR_RX_EMPTY		,	// in	: Empty flag of a TCP RX flag
	USR_RX_RE			,	// out	: TCP Rx Data Read enable
	USR_RX_RV			,	// in	: TCP Rx data valid
	USR_RX_RD			,	// in	: TCP Rx data[7:0]
	// Register Access
	REG_CLK				,	// in	: Clock
	REG_ACT				,	// in	: Access Active
	REG_DI				,	// out	: Serial Data Output
	REG_DO	,	// in	: Serial Data Input
	//DAC test
	DAC_D             , //out : DAC DATA
	DAC_SYNC             , //out : DAC SYNC
	DAC_SCLK             , //out : DAC CLK
	//ADC test
	ADC_D					,
	ADC_OTR           	,
	ADC_PDWN				,
	ADC_CLK				,
	//NIM test
	NIM_IN			,
	NIM_OUT		,
	//INTPIX STORE, RST,
	INT_STORE		,
	INT_RST		,
	INT_LEN		,
	INT_REN		,
	INT_ENRA		,
	INT_ENCA		,
	INT_RA			,
	INT_CA			,
	// SW, LED
	LED					,	// out	: LED[7:0]
	DIP_SW			,		// in	: DIP SW[7:0]
	//Jumper pin
	JUMPER
);

//-------- Input/Output -------------
	input			OSC				;
	input			SW_RSTn			;

	output			USR_CLK			;
	input			USR_ACTIVE		;
	input			USR_CLOSE_REQ	;
	output			USR_CLOSE_ACK	;
	input			USR_TX_AFULL	;
	output			USR_TX_WE		;
	output	[7:0]	USR_TX_WD		;
	input			USR_RX_EMPTY	;
	output			USR_RX_RE		;
	input			USR_RX_RV		;
	input	[7:0]	USR_RX_RD		;
	input			REG_CLK			;
	input			REG_ACT			;
	output			REG_DI			;
	input			REG_DO			;

	output   		DAC_D   	       ;
	output   		DAC_SYNC          ;
	output   		DAC_SCLK          ;
	output	 		ADC_PDWN				;
	output 		     	ADC_CLK			;
	input            	ADC_OTR			;
	input	 [11:0] 	ADC_D			;

	input	[1:0]	NIM_IN			;
	output	[1:0]	NIM_OUT		;

	output			INT_STORE		;
	output  			INT_RST		;
	output  			INT_LEN		;
	output  			INT_REN		;
	output  			INT_ENRA		;
	output  			INT_ENCA		;
	output	[6:0]	INT_RA			;
	output	[6:0]	INT_CA			;

	output	[7:0]	LED				;
	input	[7:0]	DIP_SW			;

	output  			JUMPER		;

//------------------------------------------------------------------------------
//	DLL
//------------------------------------------------------------------------------
	wire			RST_DLL				;
	wire			locked				;
	wire			CLK12_5M				;
	wire			CLK25M				;
	wire			CLK50M				;

	assign	RST_DLL		= ~SW_RSTn;

   	sys_dcm			SYSDCM(
		.CLKIN_IN			(OSC			),
		.RST_IN				(RST_DLL		),
		.CLKDV_OUT			(CLK25M			),
		.CLKIN_IBUFG_OUT	(),
		.CLK0_OUT			(CLK50M			),
		.LOCKED_OUT			(locked			)
	);

	assign	RSTn	= locked & ~RST_DLL;

	ODDR	IOB_DDRCLKp0(.Q(USR_CLK), .C(CLK25M), .CE(1'b1), .D1(1'b0), .D2(1'b1), .R( 1'b0), .S( 1'b0));

//------------------------------------------------------------------------------
//	Contents of Reg => LED
//------------------------------------------------------------------------------
	wire	[7:0]	LED				;
	reg		[7:0]	ledOut			;

	reg [7:0] regaddr;
	wire [7:0] ADDR00; //DAQ Flag 0: DAQ, 1: wait
	wire [7:0] ADDR01; //# of taking event (0-255). zero means infinity. 
	wire [7:0] ADDR02; //NIM_OUT CLK Control
	wire [7:0] ADDR03; //DAC control bit [11:8]
	wire [7:0] ADDR04; //DAC control bit [7:0]
	wire [7:0] ADDR05; //RST Hi time (unit by 40ns)
	wire [7:0] ADDR06; //Integrating time [23:16]
	wire [7:0] ADDR07; //Integrating time [15:8]
	wire [7:0] ADDR08; //Integrating time [7:0]
	wire [7:0] ADDR09; //pix scan time 0-16
	reg	[7:0] ADDR [0:9]		;	

	always@ (posedge CLK25M)begin
		if(~RSTn)begin
			regaddr <= 8'b0;
		end
		regaddr <= DIP_SW;
		ADDR[0] <= ADDR00;
		ADDR[1] <= ADDR01;
		ADDR[2] <= ADDR02;
		ADDR[3] <= ADDR03;
		ADDR[4] <= ADDR04;
		ADDR[5] <= ADDR05;
		ADDR[6] <= ADDR06;
		ADDR[7] <= ADDR07;
		ADDR[8] <= ADDR08;
		ADDR[9] <= ADDR09;
		case(regaddr)
			8'h0:	ledOut <= ADDR[0];
			8'h1:	ledOut <= ADDR[1];
			8'h2:	ledOut <= ADDR[2];
			8'h3:	ledOut <= ADDR[3];
			8'h4:	ledOut <= ADDR[4];
			8'h5:	ledOut <= ADDR[5];
			8'h6:	ledOut <= ADDR[6];
			8'h7:	ledOut <= ADDR[7];
			8'h8:	ledOut <= ADDR[8];
			8'h9:	ledOut <= ADDR[9];
			default:	ledOut <= 8'b0;
		endcase
	end		
	assign LED[6:0] = ledOut[6:0];
	reg tmpled;
	assign LED[7] = tmpled;

	wire [7:0] wireADDR3;
	wire [7:0] wireADDR4;
	wire [7:0] wireADDR9;
	assign wireADDR3 = ADDR[3];
	assign wireADDR4 = ADDR[4];
	assign wireADDR9 = ADDR[9];

//DAC test-------------------------------------
	wire DAC_D;
	wire DAC_SYNC;
	wire DAC_SCLK;

	reg [20:0] dac_cnt;
	reg [15:0] dac_buf;
	reg          dac_read;

	always @(negedge CLK25M)begin
		if(~RSTn)begin
			dac_cnt <= 21'b0;
			dac_buf <= 16'b0001_0000_0000_0000;
			dac_read <= 1'b0;
		end 
		if(dac_cnt <= 21'h11)begin
			dac_read <= 1'b0;
			if(dac_cnt >=21'h1)begin
				dac_buf <= dac_buf << 1;
			end
		end
		if(dac_cnt == 21'h1F_FFFF)begin
			dac_read <= 1'b1;
		//	dac_buf <= 16'b0001_0111_1111_1111;
			dac_buf <= {4'b0001, wireADDR3[3:0], wireADDR4};
		end
		dac_cnt <= dac_cnt + 21'b1;				
	end //always
	assign DAC_SCLK = ~CLK25M;
	assign DAC_D = dac_buf[15];
	assign DAC_SYNC = dac_read;



//TCP test-------------------------------------
	reg				irUsrActive		;
	reg				irUsrCloseReq	;
	reg				irUsrTxAfull	;
	reg				irUsrRxEmpty	;
	reg				irUsrRxRv		;
	reg		[7:0]	irUsrRxRd		;

	always@ (posedge CLK25M) begin
		irUsrActive		<= USR_ACTIVE;
		irUsrCloseReq	<= USR_CLOSE_REQ;
		irUsrTxAfull		<= USR_TX_AFULL;
		irUsrRxEmpty	<= USR_RX_EMPTY;
		irUsrRxRv		<= USR_RX_RV;
		irUsrRxRd[7:0]	<= USR_RX_RD[7:0];
		//tmpled 			<= irUsrTxAfull;
	end

	assign ADC_PDWN = 1'b0;
	assign ADC_CLK = CLK25M;

	wire [17:0] fifo_indata;
	wire [17:0] fifo_odata;
	wire fifo_we;
	wire fifo_re;
	wire fifo_empty;
	wire fifo_full;
	wire fifo_rst;

	FIFO FIFO(
		.clk (CLK25M),
		.rst (fifo_rst),
		.din (fifo_indata),
		.dout (fifo_odata),
		.rd_en (fifo_re),		
		.wr_en (fifo_we),
		.empty (fifo_empty),
		.full (fifo_full)
	);

	reg stt_no_connection;
	reg stt_wait_trig;
	reg stt_fifo_fill;
	reg stt_tcp_sending;
	reg [1:0] trig;

	reg end_fifo_fill;
	reg end_tcp_send;

	reg orInt_Store;
	reg orInt_Rst;
	reg orInt_Enca;
	reg [6:0] orInt_RA;
	reg [6:0] orInt_CA;
	reg [23:0] int_cnt;

	always @(posedge CLK25M)begin
		trig[0] <= ~NIM_IN[0];
		trig[1] <= trig[0];
		if(USR_ACTIVE==1'b0 | ~RSTn)begin
			stt_no_connection <= 1'b1;
			stt_wait_trig <= 1'b0;
			stt_fifo_fill <= 1'b0;
			stt_tcp_sending <= 1'b0;
		end else if(USR_ACTIVE == 1'b1 & ~(stt_tcp_sending == 1'b1 | stt_fifo_fill == 1'b1) & ADDR[0] == 8'b1)begin
			stt_no_connection <= 1'b0;
			stt_wait_trig <= 1'b1;
		end	
//		if(NIM_IN[0] == 1'b1 & trig[0] == 1'b1 & stt_wait_trig == 1'b1 & ~irUsrTxAfull)begin
		if(trig[0] == 1'b1 & trig[1] == 1'b0 & stt_wait_trig == 1'b1 & ~irUsrTxAfull)begin
			//trig rising edge
			stt_no_connection <= 1'b0;
			stt_tcp_sending <= 1'b1;
			stt_fifo_fill <= 1'b1;
			stt_wait_trig <= 1'b0;
		end 
		if(end_fifo_fill == 1'b1)begin
			stt_fifo_fill <= 1'b0;
		end
		if(end_tcp_send == 1'b1)begin
			stt_tcp_sending <= 1'b0;
		end
		tmpled <= stt_wait_trig;
	end//always

	reg [14:0] fifo_fill_cnt;
	reg [3:0] fifo_fill_flag;
	reg orfifo_we;
	reg orfifo_re;
	reg orfifo_rst;

	reg TCP_DATA_OK;	
	reg [15:0] send_cnt;
	reg [3:0] send_No;
	reg send_start;
	reg full_flag;

	always@ (posedge CLK25M)begin
	//initialization--------------------------
		if(USR_ACTIVE==1'b0 | ~RSTn)begin
			fifo_fill_cnt <= 15'b0;
			fifo_fill_flag <= 4'b0;
			orfifo_we <= 1'b0;
			orfifo_re <= 1'b0;
			orfifo_rst <= 1'b1;
	
		 	TCP_DATA_OK <= 1'b0;	
		 	send_cnt <= 16'b0;
	 	 	send_No <= 4'b0;
			send_start <= 1'b0;
			full_flag <= 1'b0;

			end_fifo_fill <= 1'b0;
			end_tcp_send <= 1'b0;

			orInt_Enca <= 1'b0;
		end
	//fifo fill state--------------------------
		if(stt_fifo_fill == 1'b1 & orInt_Store == 1'b0)begin
			orfifo_rst <= 1'b0;
			if(fifo_fill_flag == wireADDR9[3:0])begin
				orInt_Enca <= 1'b1;
				orInt_RA <= fifo_fill_cnt[13:7];
				orInt_CA <= fifo_fill_cnt[6:0];
				orfifo_we <= 1'b1;
				end_fifo_fill <= 1'b0;
			end else begin
				orInt_Enca <= 1'b1;
				orInt_RA <= fifo_fill_cnt[13:7];
				orInt_CA <= fifo_fill_cnt[6:0];
				orfifo_we <= 1'b0;
				end_fifo_fill <= 1'b0;
			end 
			if(fifo_fill_cnt[14:0] == 15'b100_0000_0000_0000)begin
//			if(fifo_fill_cnt[3:0] >= 3'b100)begin
				orInt_Enca <= 1'b0;
				orfifo_we <= 1'b0;
				fifo_fill_cnt <= 15'b0;
				fifo_fill_flag <= 1'b0;
				end_fifo_fill <= 1'b1;
			end else begin
				fifo_fill_flag <= fifo_fill_flag + 3'b1;
				if(fifo_fill_flag == wireADDR9[3:0])begin
					fifo_fill_cnt <= fifo_fill_cnt + 1'b1;
					fifo_fill_flag <= 4'b0;
				end
			end
			if(end_fifo_fill == 1'b1)begin
				orfifo_we <= 1'b0;
				orInt_Enca <= 1'b0;
				fifo_fill_cnt <= 15'b0;
				fifo_fill_flag <= 4'b0;
			end
		end
/*		if(stt_fifo_fill == 1'b1 & orInt_Store == 1'b0)begin
			orInt_Enca <= 1'b1;
			orInt_RA <= fifo_fill_cnt[6:0];
			orInt_CA <= fifo_fill_cnt[13:7];
			orfifo_we <= 1'b1;
			end_fifo_fill <= 1'b0;
			if(fifo_fill_cnt[14:0] >= 15'b100_0000_0000_0000)begin
			//if(fifo_fill_cnt[2:0] >= 3'b100)begin
				orInt_Enca <= 1'b0;
				orfifo_we <= 1'b0;
				fifo_fill_cnt <= 15'b0;
				end_fifo_fill <= 1'b1;
			end else begin
				fifo_fill_cnt <= fifo_fill_cnt + 1'b1;
			end
			if(end_fifo_fill == 1'b1)begin
				orInt_Enca <= 1'b0;
				orfifo_we <= 1'b0;
				fifo_fill_cnt <= 15'b0;
			end
		end
*/
	//tcp sending state----------------------
		if(stt_tcp_sending == 1'b1 & orInt_Store == 1'b0)begin
			end_tcp_send <= 1'b0;
			if(~fifo_empty)begin
				send_start <=1'b1;
			end 
			if(send_start)begin
				if(irUsrTxAfull == 1'b0 & full_flag)begin
					orfifo_re <= 1'b1;
					TCP_DATA_OK <= 1'b1;
					send_cnt <= send_cnt + 16'b1;
					full_flag <= 1'b0;
				end  else if(irUsrTxAfull == 1'b1 & send_No == 4'h2)begin
					TCP_DATA_OK <= 1'b0;
					send_No <= 4'h1;
					full_flag <= 1'b1;
				end  else if(full_flag == 1'b0)begin
				if(~(fifo_empty & send_cnt <= 16'b0111_1111_1111_1110))begin
					case(send_No)
						4'h0 : begin
							orfifo_re <= 1'b1;
							send_No <= 4'b0001;	
						end
						4'h1 : begin
							orfifo_re <= 1'b0;
							TCP_DATA_OK <= 1'b1;
							send_cnt <= send_cnt + 16'b1;
							send_No <= 4'h2;
						end
						4'h2 : begin
							orfifo_re <= 1'b1;
							TCP_DATA_OK <= 1'b1;
							send_cnt <= send_cnt + 16'b1;
							send_No <= 4'h1;
						end
					endcase
				end else begin
					TCP_DATA_OK <= 1'b0;
				end
				end
				if(send_cnt >= 16'b1000_0000_0000_0000)begin
				//if(send_cnt >= 16'b0000_0000_0000_1000)begin
					TCP_DATA_OK <= 1'b0;
					orfifo_re <= 1'b0;
					send_cnt <= 16'b0;
					end_tcp_send <= 1'b1;
					send_start <= 1'b0;
					send_No <= 1'b0;
				end 
				if(end_tcp_send == 1'b1)begin
					orfifo_re <= 1'b0;
					send_cnt <= 16'b0;
					send_start <= 1'b0;
				end
			end
		end			
	end
//For Pipeline ADC
	reg [15:0] delay_fifo_we;
	always@(posedge CLK25M)begin
		if(USR_ACTIVE == 1'b0 | ~RSTn)begin
				delay_fifo_we <= 16'b0;
		end else begin
			delay_fifo_we[15:0] <= {delay_fifo_we[14:0], orfifo_we};
		end
	end
//For waiting being AOUT static state
	reg [39:0] SR1;
	reg [11:0] SR2 [39:0];
	integer i;
	always@(posedge CLK25M)begin
		if(irUsrActive == 1'b0)begin
			SR1 <= 40'b0;
			SR2[0] <= 40'b0;
		end else begin
			SR1 <= {SR1[38:0], delay_fifo_we[6]};
			SR2[0] <= ADC_D+12'b1000_0000_0000;
			begin: LOOP
				for(i=1; i<40; i=i+1)
					SR2[i] <= SR2[i-1];
			end
		end
	end
				
//	assign fifo_indata = {3'b0, fifo_fill_cnt[14:0]};
//	assign fifo_indata = {fifo_fill_cnt[7:0], fifo_fill_cnt[7:0]};
//	assign fifo_indata = {6'b0, ADC_D+12'b1000_0000_0000};
//	assign fifo_we = delay_fifo_we[6];
	assign fifo_indata = {6'b0, SR2[39]};
	assign fifo_we = SR1[39];
	assign fifo_re = orfifo_re;
	assign fifo_rst = orfifo_rst;

//INTPIX RST & STORE	
	always@(posedge CLK25M)begin
		if(USR_ACTIVE == 1'b0 | ~RSTn)begin
			orInt_Store <= 1'b1;
			orInt_Rst <= 1'b0;
			int_cnt <= 24'b0;
		end else if(stt_wait_trig == 1'b1)begin
//			if(int_cnt < 24'b100_0000)begin
			if(int_cnt < {16'b0,ADDR[5]})begin
				orInt_Rst <= 1'b1;
				orInt_Store <= 1'b1;
				int_cnt <= int_cnt + 24'b1;	
			end else begin
				orInt_Rst <= 1'b0;
				int_cnt <= int_cnt + 24'b1;
			end
			if(orInt_Store == 1'b0)begin
				orInt_Store <= 1'b1;
				orInt_Rst <= 1'b1;
				int_cnt <= 24'b0;
			end
		end else begin
//			if(int_cnt > 24'b1_0000_0000_0000)begin
			if(int_cnt > {ADDR[8], ADDR[7], ADDR[6]})begin
				orInt_Rst <= 1'b0;
				orInt_Store <= 1'b0;
			end
			int_cnt <= int_cnt + 24'b1;
		end
//		if(int_cnt > 24'b1_0000_0000_0000)begin
		if(int_cnt > {ADDR[8], ADDR[7], ADDR[6]})begin
			int_cnt <= 24'b0;
		end
	end

	assign INT_STORE = orInt_Store;
	assign INT_RST = orInt_Rst;
	
	assign INT_REN = 1'b0;
	assign INT_LEN = 1'b0;
	assign INT_ENRA = 1'b1;
	assign INT_ENCA = 1'b1;
//	assign INT_RA   = 7'd63;
//	assign INT_CA   = 7'd63;
	
// assign INT_ENCA = orInt_Enca;
	assign INT_RA = orInt_RA;
	assign INT_CA = orInt_CA;


//------------------------------------------------------------------------------
//NIM test-------------------------------------
	reg [23:0] clk_cnt ;
	reg NIM_CLK;
	always@ (posedge CLK25M)begin
		if(~irUsrActive)begin
			clk_cnt <= 24'b0;
		end
		if(stt_wait_trig)begin
			clk_cnt <= clk_cnt + 24'b1;
			case(ADDR[2])
				8'b00:	NIM_CLK <= clk_cnt[23];
				8'b01:	NIM_CLK <= clk_cnt[22];
				8'b10:	NIM_CLK <= clk_cnt[21];
				8'b11:	NIM_CLK <= clk_cnt[20];
				8'b100:	NIM_CLK <= clk_cnt[19];
				8'b101:	NIM_CLK <= clk_cnt[18];
				8'b110:	NIM_CLK <= clk_cnt[17];
				8'b111:	NIM_CLK <= clk_cnt[16];
				8'b1000: NIM_CLK <= 1'b0;
				default:  NIM_CLK <= clk_cnt[23];
			endcase 
		end
	end
	wire [1:0] NIM_OUT;
	assign NIM_OUT[0] = NIM_CLK;
//	assign NIM_OUT[1] = ~stt_wait_trig;

	assign NIM_OUT[1] = orInt_Rst;
	
	reg delay_rst;
	always@ (posedge CLK25M)begin
		if(~irUsrActive)begin
			delay_rst <= 1'b0;
		end else begin
			delay_rst <= NIM_IN[1];
		end
	end
	assign JUMPER = ~delay_rst; 

//------------------------------------------------------------------------------

	wire [1:0] Number;
	assign Number = send_No[1:0];	

	reg [1:0] sndNumber;
	reg usrTxWe		;
	reg [15:0] usrTxWd		;
	reg usrRxRe		;


	always@ (posedge CLK25M) begin
		usrTxWe		<=  irUsrActive & ~irUsrCloseReq & TCP_DATA_OK;
		usrTxWd[15:0]  	<=  fifo_odata[15:0];
		usrRxRe		<=  irUsrActive & ~irUsrCloseReq & ~irUsrRxEmpty;
		sndNumber 		<=  Number;
	end

	reg				orUsrCloseAck	;
	reg				orUsrTxWe		;
	reg		[7:0]	orUsrTxWd		;
	reg				orUsrRxRe		;

	always@ (posedge CLK25M) begin
		orUsrCloseAck	<= irUsrCloseReq;
		orUsrTxWe		<= usrTxWe;
		orUsrRxRe		<= usrRxRe;
		if(sndNumber == 2'h2)begin
//			orUsrTxWd[7:0]	<= usrTxWd[15:8];
			orUsrTxWd[7:0]	<= usrTxWd[7:0];
		end else begin
//			orUsrTxWd[7:0]	<= usrTxWd[7:0];
			orUsrTxWd[7:0]	<= usrTxWd[15:8];
		end
	end

	reg TCP_WE;
	always@ (posedge CLK25M)begin
		TCP_WE <= orUsrTxWe;
	end

	assign	USR_CLOSE_ACK	= orUsrCloseAck;
	assign	USR_TX_WE			= orUsrTxWe;
//	assign	USR_TX_WE			= TCP_WE;
	assign	USR_TX_WD			= orUsrTxWd[7:0];
	assign	USR_RX_RE			= orUsrRxRe;

//------------------------------------------------------------------------------
//	Register Access
//------------------------------------------------------------------------------
	wire	[31:0]	SIO_ADDR		;
	wire	[7:0]	SIO_WD			;
	wire			SIO_WE			;
	wire			SIO_RE			;
	wire			SIO_ACK			;
	wire			SIO_RV			;
	wire	[7:0]	SIO_RD			;


	SIO_SLAVE			SIO_SLAVE(
		.RSTn					(RSTn					),	// in	: System reset
		.FILL_ADDR				(32'h0					),	// in	: Filled address for narow address-width
		// Serial I/F
		.SCK					(REG_CLK				),	// in	: Clock
		.SCS					(REG_ACT				),	// in	: Active
		.SI						(REG_DI					),	// out	: Data input
		.SO						(REG_DO					),	// in	: Data output
		// Register I/F
		.REG_ADDR				(SIO_ADDR[31:0]			),	// out	: Address[31:0]
		.REG_WD					(SIO_WD[7:0]			),	// out	: Data[7:0]
		.REG_WE					(SIO_WE					),	// out	: Write enable
		.REG_RE					(SIO_RE					),	// out	: Read enable
		.REG_ACK				(SIO_ACK				),	// in	: Access acknowledge
		.REG_RV					(SIO_RV					),	// in	: Read valid
		.REG_RD					(SIO_RD[7:0]			)	// in	: Read data[7:0]
	);

	SIO_REG				SIO_REG(
		.SCK					(REG_CLK				),	// in	: Clock
		.RSTn					(RSTn					),	// in	: System reset
		//test by hirose
		.ADDR00						(ADDR00				),
		.ADDR01						(ADDR01				),
		.ADDR02						(ADDR02				),
		.ADDR03						(ADDR03				),
		.ADDR04						(ADDR04				),
		.ADDR05						(ADDR05				),
		.ADDR06						(ADDR06				),
		.ADDR07						(ADDR07				),
		.ADDR08						(ADDR08				),
		.ADDR09						(ADDR09				),

		// Register I/F	
		.REG_ADDR				(SIO_ADDR[31:0]			),	// in	: Address[31:0]
		.REG_WD					(SIO_WD[7:0]			),	// in	: Data[7:0]
		.REG_WE					(SIO_WE					),	// in	: Write enable
		.REG_RE					(SIO_RE					),	// in	: Read enable
		.REG_ACK				(SIO_ACK				),	// out	: Access acknowledge
		.REG_RV					(SIO_RV					),	// out	: Read valid
		.REG_RD					(SIO_RD[7:0]			)	// out	: Read data[7:0]

	);

		
//------------------------------------------------------------------------------
endmodule